import {
  Controller,
  Get,
  Query,
  Headers,
  Body,
  Post,
  NotFoundException,
} from '@nestjs/common';
import { ProductsService } from './products.service';
import { DistanceUnitEnum, ParcelCreateRequest, WeightUnitEnum } from 'shippo';
import { ShippoService } from '../shippo/shippo.service';
import { StripeService } from '../stripe/stripe.service';
import { PreOrdersService } from '../orders/pre-orders.service';

@Controller('products')
export class ProductsController {
  constructor(
    private productsService: ProductsService,
    private shippoService: ShippoService,
    private stripeService: StripeService,
    private preOrdersService: PreOrdersService,
  ) {}

  @Get('')
  getAll() {
    return this.productsService.getAll();
  }

  @Get('get-shipping-rates')
  async getShippingRates(
    @Query('addressTo') addressTo: string,
    @Query('items') items: string,
  ) {
    // Our store address
    const addressFrom = {
      name: 'Mini Shop',
      street1: '4727 W 1st St',
      city: 'New York',
      state: 'NY',
      zip: '10005',
      country: 'US',
    };

    // The user address
    const addressToObj = JSON.parse(addressTo);
    const parcels = await this.getParcels(items);
    return this.shippoService.getRates(addressFrom, addressToObj, parcels);
  }

  async getParcels(items: string) {
    const parcels: ParcelCreateRequest[] = [];
    const itemsObj = JSON.parse(items);
    const ids = itemsObj.map((t) => t.id);
    const products = await this.productsService.getProducts(ids);
    products.forEach((p) => {
      parcels.push({
        length: p.length.toString(),
        width: p.width.toString(),
        height: p.height.toString(),
        weight: p.weight.toString(),
        massUnit: p.massUnit as WeightUnitEnum,
        distanceUnit: p.distanceUnit as DistanceUnitEnum,
      });
    });
    return parcels;
  }

  @Post('checkout')
  async Checkout(
    @Body('cartItems') cartItems,
    @Body('rate') rate,
    @Headers() headers,
  ) {
    const products = await this.productsService.getProducts(
      cartItems.map((t) => t.id),
    );
    if (products && products.length > 0) {
      const itemsArr = products.map((product) => ({
        id: product.id,
        name: product.title,
        price: product.price,
        imageUrl: product.imageUrl,
        quantity:
          cartItems.find((item) => item.id === product.id)?.quantity ?? 1,
      }));
      let total = 0;
      const userId = 1;
      itemsArr.forEach((item) => {
        total += Math.round(item.price * item.quantity);
      });
      const preOrder = await this.preOrdersService.create({
        items: JSON.stringify(itemsArr),
        orderDate: new Date(),
        rate,
        total,
        userId,
      });
      return this.stripeService.checkout(preOrder, headers);
    } else throw new NotFoundException('Products not found!');
  }
}
