import { Component, computed, OnInit } from '@angular/core';
import { FormBuilder, ReactiveFormsModule, Validators } from '@angular/forms';
import { first, forkJoin } from 'rxjs';
import { StarComponent } from '../../components/star/star.component';
import { ButtonModule } from 'primeng/button';
import { InputTextModule } from 'primeng/inputtext';
import { ProductsService } from '../../services/products.service';
import { CartService } from '../../services/cart.service';
import { PanelModule } from 'primeng/panel';
import { ShippingRateCardComponent } from '../../components/shipping-rate-card/shipping-rate-card.component';
import { LoadingComponent } from '../../components/loading/loading.component';
import { DividerModule } from 'primeng/divider';
import { Address } from '../../data/address';
import { loadStripe } from '@stripe/stripe-js/pure';
import { environment } from '../../../environments/environment';
import { SummaryShippingRateCardComponent } from '../../components/summary-shipping-rate-card/summary-shipping-rate-card.component';
import { CommonModule } from '@angular/common';

@Component({
  selector: 'app-shipping',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    StarComponent,
    InputTextModule,
    ButtonModule,
    PanelModule,
    ShippingRateCardComponent,
    SummaryShippingRateCardComponent,
    LoadingComponent,
    DividerModule,
    CommonModule,
  ],
  templateUrl: './shipping.component.html',
  styleUrl: './shipping.component.css',
})
export class ShippingComponent implements OnInit {
  dimensions = {};
  address: Address | undefined;
  rates: any[] = [];
  selectedRate: any;
  isLoading = false;
  featuredRates: any[] = [];
  otherRates: any[] = [];
  bestValue: any;
  fastest: any;
  cheapest: any;
  selectedRateAmount = 0;
  count = computed(() => this.cartService.cart().count);
  total = computed(() => this.cartService.cart().total);

  constructor(
    private productsService: ProductsService,
    private cartService: CartService
  ) {}

  ngOnInit(): void {
    this.address = {
      name: 'John Smith',
      street1: '456 Another Ave',
      city: 'Los Angeles',
      state: 'CA',
      zip: '90001',
      country: 'US',
    };
    this.loadShippingMethods();
  }

  loadShippingMethods() {
    this.isLoading = true;
    this.productsService
      .getShippingRates(
        JSON.stringify(this.address),
        JSON.stringify(
          this.cartService.cart().items.map((t) => ({
            id: t.id,
            quantity: t.quantity,
          }))
        )
      )
      .pipe(first())
      .subscribe({
        next: (t: any) => {
          if (t && t.rates && t.rates.length > 0) {
            this.rates = t.rates;
            this.bestValue = this.rates.find(
              (rate: any) => rate.attributes[0] === 'BESTVALUE'
            );
            this.fastest = this.rates.find(
              (rate: any) => rate.attributes[0] === 'FASTEST'
            );
            this.cheapest = this.rates.find(
              (rate: any) => rate.attributes[0] === 'CHEAPEST'
            );
            this.rates.forEach((rate: any) => {
              if (
                rate.objectId !== this.bestValue?.objectId &&
                rate.objectId !== this.fastest?.objectId &&
                rate.objectId !== this.cheapest?.objectId
              ) {
                this.otherRates.push(rate);
              }
            });
          }
          this.isLoading = false;
        },
        error: (e: any) => {
          this.isLoading = false;
        },
      });
  }

  onSelectRate(rate: any) {
    this.selectedRate = rate;
    this.selectedRateAmount = parseFloat(rate.amount);
  }

  onCheckout() {
    if (this.selectedRate) {
      const cartItems = this.cartService
        .cart()
        .items.map((item) => ({ id: item.id, quantity: item.quantity }));
      this.productsService
        .checkout(cartItems, this.selectedRate)
        .pipe(first())
        .subscribe({
          next: async (r: any) => {
            const session = r as any;
            const stripe = await loadStripe(environment.STRIPE_PK);
            const result = await stripe?.redirectToCheckout({
              sessionId: session.id,
            });
            if (result?.error) {
              console.log(result?.error);
            }
            this.cartService.clear();
          },
          error: (e: any) => {},
        });
    }
  }
}
